@@ -22,12 +22,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 module Text.TeXMath ( texMathToMathML, DisplayType(..) )
 where
 import Text.TeXMath.Parser
-import Text.TeXMath.MathMLWriter
+import Text.TeXMath.MathML
 import Text.XML.Light
-import Text.ParserCombinators.Parsec (parse)
 
 texMathToMathML :: DisplayType -> String -> Either String Element
 texMathToMathML dt inp = inp `seq`
-  case parse formula "TeX math input" inp of
-       Left err        -> Left (show err)
-       Right v         -> Right $ toMathML dt v
+  either Left (Right . toMathML dt) $ parseFormula inp
