@@ -1,4 +1,3 @@
-{-# LANGUAGE DeriveDataTypeable #-}
 {-
 Copyright (C) 2009 John MacFarlane <jgm@berkeley.edu>
 
@@ -20,7 +19,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 {- | Functions for parsing a LaTeX formula to a Haskell representation.
 -}
 
-module Text.TeXMath.Parser (expr, formula, Exp(..), TeXSymbolType(..), ArrayLine, Alignment(..))
+module Text.TeXMath.Parser (parseFormula)
 where
 
 import Control.Monad
@@ -29,39 +28,7 @@ import qualified Data.Map as M
 import Text.ParserCombinators.Parsec
 import qualified Text.ParserCombinators.Parsec.Token as P
 import Text.ParserCombinators.Parsec.Language
-import Data.Generics
-
-data TeXSymbolType = Ord | Op | Bin | Rel | Open | Close | Pun | Accent
-                     deriving (Show, Read, Eq, Data, Typeable)
-
-data Alignment = AlignLeft | AlignCenter | AlignRight | AlignDefault
-                 deriving (Show, Read, Eq, Data, Typeable)
-
-type ArrayLine = [[Exp]]
-
-data Exp =
-    ENumber String
-  | EGrouped [Exp]
-  | EIdentifier String
-  | EMathOperator String
-  | ESymbol TeXSymbolType String
-  | ESpace String
-  | EBinary String Exp Exp
-  | ESub Exp Exp
-  | ESuper Exp Exp
-  | ESubsup Exp Exp Exp
-  | EOver Exp Exp
-  | EUnder Exp Exp
-  | EUnderover Exp Exp Exp
-  | EUp Exp Exp
-  | EDown Exp Exp
-  | EDownup Exp Exp Exp
-  | EUnary String Exp
-  | EScaled String Exp
-  | EStretchy Exp
-  | EArray [Alignment] [ArrayLine]
-  | EText String String
-  deriving (Show, Read, Eq, Data, Typeable)
+import Text.TeXMath.Types
 
 texMathDef :: LanguageDef st
 texMathDef = LanguageDef 
@@ -95,8 +62,14 @@ expr1 =  choice [
   , diacritical
   , escaped
   , unicode
+  , ensuremath
   ]
 
+-- | Parse a formula, returning a list of 'Exp'.
+parseFormula :: String -> Either String [Exp]
+parseFormula inp =
+  either (Left . show) (Right . id) $ parse formula "formula" inp
+
 formula :: GenParser Char st [Exp]
 formula = do
   whiteSpace
@@ -297,6 +270,10 @@ escaped = lexeme $ try $
 unicode :: GenParser Char st Exp
 unicode = lexeme $ liftM (ESymbol Ord . (:[])) $ satisfy (not . isAscii)
 
+ensuremath :: GenParser Char st Exp
+ensuremath = lexeme $ try $
+             string "\\ensuremath" >> inbraces
+
 command :: GenParser Char st String
 command = try $ char '\\' >> liftM ('\\':) (identifier <|> lexeme (count 1 anyChar))
 
@@ -305,14 +282,14 @@ unaryOps = ["\\sqrt", "\\surd"]
 
 textOps :: M.Map String (String -> Exp)
 textOps = M.fromList
-          [ ("\\textrm", EText "normal")
+          [ ("\\textrm", EText "normal" . parseText)
           , ("\\mathrm", EText "normal")
-          , ("\\text",   EText "normal")
+          , ("\\text",   EText "normal" . parseText)
           , ("\\mbox",   EText "normal")
           , ("\\mathbf", EText "bold")
-          , ("\\textbf", EText "bold")
+          , ("\\textbf", EText "bold" . parseText)
           , ("\\mathit", EText "italic")
-          , ("\\textit", EText "italic")
+          , ("\\textit", EText "italic" . parseText)
           , ("\\mathtt", EText "monospace")
           , ("\\texttt", EText "monospace")
           , ("\\mathsf", EText "sans-serif")
@@ -321,6 +298,17 @@ textOps = M.fromList
           , ("\\mathfrak", EText "fraktur")
           ]
 
+parseText :: String -> String
+parseText ('`':'`':xs) = '\x201C' : parseText xs
+parseText ('\'':'\'':xs) = '\x201D' : parseText xs
+parseText ('\'':xs) = '\x2019' : parseText xs
+parseText ('-':'-':'-':xs) = '\x2014' : parseText xs
+parseText ('-':'-':xs) = '\x2013' : parseText xs
+parseText ('\\':'l':'d':'o':'t':'s':xs) = '\x2026' : parseText xs
+parseText ('~':xs) = '\xA0' : parseText xs
+parseText (x:xs) = x : parseText xs
+parseText [] = []
+
 diacritical :: GenParser Char st Exp
 diacritical = try $ do
   c <- command
@@ -447,7 +435,7 @@ enclosures = [ ("(", ESymbol Open "(")
              , ("\\rbrack", ESymbol Close "]")
              , ("\\rbrace", ESymbol Close "}")
              , ("\\llbracket", ESymbol Open "\x27E6")
-             , ("\\rrbracket", ESymbol Close "\x230B")
+             , ("\\rrbracket", ESymbol Close "\x27E7")
              , ("\\langle", ESymbol Open "\x27E8")
              , ("\\rangle", ESymbol Close "\x27E9")
              , ("\\lfloor", ESymbol Open "\x230A")
@@ -458,10 +446,14 @@ enclosures = [ ("(", ESymbol Open "(")
              , ("|", ESymbol Close "\x2223")
              , ("\\|", ESymbol Open "\x2225")
              , ("\\|", ESymbol Close "\x2225")
-             , ("\\vert", ESymbol Open "\x2223")
-             , ("\\vert", ESymbol Close "\x2223")
-             , ("\\Vert", ESymbol Open "\x2225")
-             , ("\\Vert", ESymbol Close "\x2225")
+             , ("\\lvert", ESymbol Open "\x7C")
+             , ("\\rvert", ESymbol Close "\x7C")
+             , ("\\vert", ESymbol Close "\x7C")
+             , ("\\lVert", ESymbol Open "\x2225")
+             , ("\\rVert", ESymbol Close "\x2225")
+             , ("\\Vert", ESymbol Close "\x2016")
+             , ("\\ulcorner", ESymbol Open "\x231C")
+             , ("\\urcorner", ESymbol Close "\x231D")
              ]
 
 symbols :: M.Map String Exp
@@ -494,7 +486,7 @@ symbols = M.fromList [
            , ("\\chi", ESymbol Ord "\x03C7")
            , ("\\delta", ESymbol Ord "\x03B4")
            , ("\\Delta", ESymbol Op "\x0394")
-           , ("\\epsi", ESymbol Ord "\x03B5")
+           , ("\\epsilon", ESymbol Ord "\x03B5")
            , ("\\varepsilon", ESymbol Ord "\x025B")
            , ("\\eta", ESymbol Ord "\x03B7")
            , ("\\gamma", ESymbol Ord "\x03B3")
@@ -684,6 +676,7 @@ symbols = M.fromList [
            , ("\\angle", ESymbol Ord "\x2220")
            , ("\\diamond", ESymbol Op "\x22C4")
            , ("\\Diamond", ESymbol Op "\x25C7")
+           , ("\\lozenge", ESymbol Op "\x25CA")
            , ("\\neg", ESymbol Op "\x00AC")
            , ("\\lnot", ESymbol Ord "\x00AC")
            , ("\\bot", ESymbol Ord "\x22A5")
